/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Soyapi Mumba.
 *
 * Portions created by the Initial Developer are Copyright (C) 2005,2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
var SearchWith = {
    
    checkFirstRun: function() {
        var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService);  
        var thisPrefBranch = prefs.getBranch("");
        
        var firstRun = false;
        try {
            firstRun = thisPrefBranch.getBoolPref("extensions.searchwith.firstRun");
        } catch(e) {
        }
        
        if (firstRun) {
            SearchWith.doFirstRun();
        }
		try {
			thisPrefBranch.setBoolPref("extensions.searchwith.firstRun", false);
		} catch(e) {
		}
        return;
    },
    
    doFirstRun: function() {
	
		if (navigator.userAgent.search(/Thunderbird/gi) != -1) {
			return; // as of now, nothing to do when we're not in a browser
		}
        var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService);  
        var thisPrefBranch = prefs.getBranch("");
        var swDefaultBranch = prefs.getDefaultBranch("extensions.searchwith.");

		this.importSearchBarEngines();
        if (navigator.userAgent.search(/Flock/gi) != -1) {
            var browserBookmarkEngineId = "";
            try {
                browserBookmarkEngineId = thisPrefBranch.getComplexValue("flock.favorites.webservice.id",
                                    Components.interfaces.nsISupportsString).data;
                //swDefaultBranch.setCharPref("service-list.bookmark", browserBookmarkEngineId);
                thisPrefBranch.setCharPref("extensions.searchwith.service-list.bookmark", 
                                            browserBookmarkEngineId);
            } catch(e) {
            }
			
			// use Flock's sourceid parameter
			var allEngines = SearchWith.getEngines();
			for (each in allEngines) {
				var theEngine = new SWEngine(allEngines[each]);
				var newParams = theEngine.getRightUrl().replace(/sourceid=mozilla/gi, "sourceid=flock-search");
				theEngine.setRightUrl(newParams);
			}
        }
        
        // get browser's default web search engine
        var browserWebEngineId = SearchWith.getPropString("browser.search.defaultenginename", 
                                            "browserBundle").toLowerCase();
        var thisEngine = new SWEngine(browserWebEngineId);
        if (thisEngine.getName().length > 0) {
			try {
				thisPrefBranch.setCharPref("extensions.searchwith.service-list.web", 
                                            browserWebEngineId);
			} catch(e) {
			}
        }
        
    },
    
    // enable SearchWith menu only when some text has been selected
    initMenu: function() 
    {
        var swServices = this.getServices();		
        var selText = this.getSelectedText();
        if (selText.length < 1) {
            document.getElementById("context-searchwith").setAttribute("hidden", "true");
            return;
        } 
        var swMenu = document.getElementById("context-searchwith");
        var menuItem;
        var cmdStr;
        var swPopup = document.getElementById("popup-searchwith");

        swMenu.setAttribute("hidden", "false");
        this.removeChildren(swPopup);

        menuItem = this.getSWMenuItem(swPopup, this.getPropString("address.bar", 
                                        "searchwith-strings"), "url");
        swPopup.appendChild(menuItem);
        menuItem = document.createElement("menuseparator");
        swPopup.appendChild(menuItem);

        for (swService in swServices) {
            var searchService = new SWService(swServices[swService]);
            var anEngineId = searchService.getEngineId();
            
            if (anEngineId.length<1) {
                continue;
            }
            
            var anEngine = new SWEngine(anEngineId);
            
            menuItem = this.getSWMenuItem(swPopup, searchService.getName(), swServices[swService]);
            menuItem.setAttribute("class", "menuitem-iconic");
            menuItem.setAttribute("image", anEngine.getIconPath());
            swPopup.appendChild(menuItem);
        }
        
        swPopup.appendChild(document.createElement("menuseparator"));
		swPopup.appendChild(this.getSWMenuItem(swPopup, this.getPropString("all", 
                                                        "searchwith-strings"), "all"));
		
		menuItem = this.getSWMenuItem(swPopup, 
                this.getPropString("manage.services", "searchwith-strings"), "options");
        menuItem.setAttribute("oncommand", 
			'window.openDialog("chrome://searchwith/content/swOptions.xul","Options","chrome");');
        swPopup.appendChild(menuItem);
    },
      
    getSWMenuItem: function(aPopup, aLabel, aValue) 
    {
        var aMenuItem = null;
        aMenuItem = document.createElement("menuitem");
        aMenuItem.setAttribute("id", "sw-"+aValue);
        aMenuItem.setAttribute("label", aLabel);
        aMenuItem.setAttribute("oncommand", "SearchWith.search('"+aValue+"', event);");
        return aMenuItem;
    },
    
    search: function(engineId, thisEvent) 
    {
		var thisApp = null;
		if (navigator.userAgent.search(/Thunderbird/gi) != -1) {
			thisApp = Components.classes["@mozilla.org/messenger;1"].createInstance();         
            thisApp = thisApp.QueryInterface(Components.interfaces.nsIMessenger);
		} else {
			thisApp = getBrowser();
		}
		
        selText = this.getSelectedText();
        if (selText.length < 1) {
            alert(SearchWith.getPropString("select.text.first", "searchwith-strings"));
            return;
        }
        
        if (thisEvent.shiftKey) {  // advanced search when shift key is pressed
            window.openDialog("chrome://searchwith/content/swEditSelection.xul","Edit Selection",
                      "chrome",selText, engineId, thisApp);
        } else {
            this.chooseEngine(selText, engineId, thisApp);
        }
    },
    
    chooseEngine: function(selText, aServiceId, aBrowser) 
    {
		selText = selText.replace(/\s+/g, "%20");
        if (aServiceId == "url") {
            if (selText.search(/:\/\//) == -1) {
                selText = "http:"+selText;
            }
            this.openBrowserTab(selText, aBrowser);
            return;
        }

        var swServices = this.getServices();
        var anEngine = "";
        var anEngineId = "";
        
        if (aServiceId == "all") {
            for (swService in swServices) {
                aService = new SWService(swServices[swService]);
                anEngine = new SWEngine(aService.getEngineId());
                if (aService.getEngineId().length>0) {
                    this.openBrowserTab(anEngine.getLeftUrl() + selText + 
                                        anEngine.getRightUrl(), aBrowser);
                }
            }
        } else {
            aService = new SWService(aServiceId);
            anEngine = new SWEngine(aService.getEngineId());
            this.openBrowserTab(anEngine.getLeftUrl() + selText + 
                                anEngine.getRightUrl(), aBrowser);
        }
    },
    
    getSelectedText: function() 
    {
        var node = document.popupNode;
        var selText = "";
    
        if ((node instanceof HTMLTextAreaElement) || (node instanceof HTMLInputElement && node.type == "text")) {
            selText = node.value.substring(node.selectionStart, node.selectionEnd);
            selText = selText.trim();
        } 
        else {
            var focusedWindow = new XPCNativeWrapper(document.commandDispatcher.focusedWindow, 'document', 'getSelection()');
            selText = focusedWindow.getSelection().toString();
            selText = selText.trim();
        }
        
        if (selText.length <1) {
            selText = content.document.getSelection().trim();
        }
        
        return selText;
    },
    
    openBrowserTab: function(srchUrl, anApp) 
    {
		if (navigator.userAgent.search(/Thunderbird/gi) != -1) {
            anApp.launchExternalURL(srchUrl);
        } else {
            anApp.selectedTab = anApp.addTab(srchUrl);
        }
    },
    
    removeChildren: function(anElement) 
    {
        while (anElement.hasChildNodes()) {
            anElement.removeChild(anElement.firstChild);
        }
    },
    
    getPropString: function(strName, aBundleName)
    {
        try {
            var strBundle = document.getElementById(aBundleName);
            return strBundle.getString(strName);
        } catch(e) {
            return strName;
        }
    },
	
	getServices: function()
	{
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService);  
        var swPrefs = prefs.getBranch("extensions.searchwith.");
        var svcPrefs = prefs.getBranch("extensions.searchwith.service-list.");				
        var swServices = svcPrefs.getChildList("", {});		
        
		swServices.sort();
		return swServices;
	},
	
	getEngines: function()
	{
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService);  
        var swPrefs = prefs.getBranch("extensions.searchwith.");
        var svcPrefs = prefs.getBranch("extensions.searchwith.engine-list.");				
        var swEngines = svcPrefs.getChildList("", {});		
		
        swEngines.sort();
		return swEngines;
	},
    
    createId: function(aName) {
        var newId = aName.trim();
        if (newId.length <1) {
            return "";
        }
        
        newId = newId.toLowerCase();
        newId = newId.replace(/\W+/g, '');
        return newId;
    },
	
	resetSettings: function() {
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService);  
        var swPrefBranch = prefs.getBranch("extensions.searchwith.");
		var swDefaultBranch = prefs.getDefaultBranch("extensions.searchwith.");
		var swSettings = swPrefBranch.getChildList("",{});
    
		for (each in swSettings) {
			try {
                swPrefBranch.clearUserPref(swSettings[each]);
			} catch(e) {
			}
		}
	},
	
	importSearchBarEngines: function() 
	{
        // adapted for SearchWith from ConQuery's FillMenu (conquery.js)
		cqrPlugsDir = GetMycroftDir();
        cqrPlugsDir2 = cqrGetProfilePluginsDir();
        var entry = new Object();

        plg_QuickLoadAll();
        var entries = EntryHash.Items;
        
        for (each in entries) {
            entry = entries[each];
            QueryHash = new Object();
            //if plugin not loaded - load it
            if (!entry.Inputs){
                entry = plg_ReadData(entry.filepath);
                EntryHash._AddEntryObj(entry, entry.eId);
            }
            
            PrepareAction(entry);
            if (entry.Action){
                var cqrurl = cqrAction;
            } else {
                var cqrurl = entry.Action;
            }

            firstpass = true;
            for (each in QueryHash){
                if (firstpass){
                    cqrurl = cqrurl + "?" + each + "=" + QueryHash[each];
                    firstpass = false;
                } else {
                    cqrurl = cqrurl + "&" + each + "=" + QueryHash[each];
                }
            }
            var arr = cqrurl.match(/(.*)(\[:selection\])(.*)/);
            
            var swEngine = new SWEngine(SearchWith.createId(entry.Title));
            swEngine.setName(entry.Title);
            swEngine.setLeftUrl(arr[1]);
            swEngine.setRightUrl(arr[3]);
            
            // support http:// and https:// paths for icons
			if (entry.iconpath.toLowerCase().substr(0,4) != "http") {
                entry.iconpath = "file://"+entry.iconpath;
            }
            swEngine.setIconPath(entry.iconpath);
		}
	}

};


// Add trim() method to String Class
String.prototype.trim = function() 
{ 
    return this.replace(/^\s+|\s+$/g, ''); 
};

window.addEventListener("load", SearchWith.checkFirstRun, false);
